/* Albert Chuang, 2004-11-03
 * read, write mac address 
 * WAN_MAC
 * LAN_MAC
 * WIRELESS_MAC
 * MAC address length is 6 bytes
 * size 0x0400(32 bytes) in config tail
 */

#include <stdio.h>
#include <stdlib.h>
#include "profile.h"

int read_mac(enum MAC_TYPE type, char *mac);
int write_mac(enum MAC_TYPE type, char *mac);
int read_mac_from_flash(HW_DATA_T *hw_data);
int write_mac_to_flash(HW_DATA_T *hw_data);
void print_mac(char *mac);
void show_usage();


#define DBG_WHERE  fprintf(stderr, __FILE__ " %d\n",  __LINE__);
//#define DBG_WHERE   
#define DBG_P(args...) fprintf(stderr, args);

void print_mac(char *mac)
{
	int i;
	for(i=0;i<MAC_LENGTH;i++){
		DBG_P("%02hhX ", mac[i]);
	}
}


int read_mac(enum MAC_TYPE type, char *mac)
{
	HW_DATA_T hw_data;
	int err=0;
	
	err = read_mac_from_flash(&hw_data);
	if(err) return -1;
		
	switch(type){
	case LAN_MAC:
		//strncpy(mac, hw_data.lan_mac, MAC_LENGTH);
		mac[0]=hw_data.lan_mac[0];
		mac[1]=hw_data.lan_mac[1];
		mac[2]=hw_data.lan_mac[2];
		mac[3]=hw_data.lan_mac[3];
		mac[4]=hw_data.lan_mac[4];
		mac[5]=hw_data.lan_mac[5];
		break;
	case WIRELESS_MAC:
		//strncpy(mac, hw_data.wireless_mac, MAC_LENGTH);
		mac[0]=hw_data.wireless_mac[0];
		mac[1]=hw_data.wireless_mac[1];
		mac[2]=hw_data.wireless_mac[2];
		mac[3]=hw_data.wireless_mac[3];
		mac[4]=hw_data.wireless_mac[4];
		mac[5]=hw_data.wireless_mac[5];
		break;
	case WAN_MAC:
		//strncpy(mac, hw_data.wan_mac, MAC_LENGTH);
		mac[0]=hw_data.wan_mac[0];
		mac[1]=hw_data.wan_mac[1];
		mac[2]=hw_data.wan_mac[2];
		mac[3]=hw_data.wan_mac[3];
		mac[4]=hw_data.wan_mac[4];
		mac[5]=hw_data.wan_mac[5];
		break;
	default:
		fprintf(stderr,"read_mac: unknow mac type\n");
	}
	return 0;
}

int write_mac(enum MAC_TYPE type, char *mac)
{
	HW_DATA_T hw_data;
	int err=0;
	
	err = read_mac_from_flash(&hw_data);
	if(err) return -1;

	switch(type){
	case LAN_MAC:
		//strncpy(hw_data.lan_mac, mac, MAC_LENGTH);
		hw_data.lan_mac[0]=mac[0];
		hw_data.lan_mac[1]=mac[1];
		hw_data.lan_mac[2]=mac[2];
		hw_data.lan_mac[3]=mac[3];
		hw_data.lan_mac[4]=mac[4];
		hw_data.lan_mac[5]=mac[5];
		write_mac_to_flash(&hw_data);
		break;
	case WIRELESS_MAC:
		//strncpy(hw_data.wireless_mac, mac, MAC_LENGTH);
		hw_data.wireless_mac[0]=mac[0];
		hw_data.wireless_mac[1]=mac[1];
		hw_data.wireless_mac[2]=mac[2];
		hw_data.wireless_mac[3]=mac[3];
		hw_data.wireless_mac[4]=mac[4];
		hw_data.wireless_mac[5]=mac[5];
		write_mac_to_flash(&hw_data);
		break;
	case WAN_MAC:
		//strncpy(hw_data.wan_mac, mac, MAC_LENGTH);
		hw_data.wan_mac[0]=mac[0];
		hw_data.wan_mac[1]=mac[1];
		hw_data.wan_mac[2]=mac[2];
		hw_data.wan_mac[3]=mac[3];
		hw_data.wan_mac[4]=mac[4];
		hw_data.wan_mac[5]=mac[5];
		write_mac_to_flash(&hw_data);
		break;
	default:
		fprintf(stderr,"write_mac: unknow mac type\n");
	}
	return 0;
}

int read_mac_from_flash(HW_DATA_T *hw_data)
{
	FILE *fp;
	
	fp = fopen(FLASH_DEVICE, "rb");
	if(fp == NULL) {
		fprintf(stderr,"read_mac_from_flash: can't open %s\n", FLASH_DEVICE);
		return -1;
	}
	
	fseek(fp, -OFFSET_MAC_ADDR, SEEK_END);
	fread(hw_data, sizeof(HW_DATA_T), 1, fp);
	
	fclose(fp);
	
	return 0;
}

int write_mac_to_flash(HW_DATA_T *hw_data)
{
	FILE *fp;
	
	fp = fopen(FLASH_DEVICE, "w+b");
	if(fp == NULL) {
		fprintf(stderr,"write_mac_from_flash: can't open %s\n", FLASH_DEVICE);
		return -1;
	}
	
	fseek(fp, -OFFSET_MAC_ADDR, SEEK_END);
	fwrite(hw_data, sizeof(HW_DATA_T), 1, fp);
	fclose(fp);
	
	return 0;
}

int show_lan_mac()
{
	char mac[6];
	int err=0;
	
	err = read_mac(LAN_MAC, mac);
	if(err) return -1;

	fprintf(stderr, "LAN=%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n",
		mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	return 0;
}

int show_wan_mac()
{
	char mac[6];
	int err=0;
	
	err = read_mac(WAN_MAC, mac);
	if(err) return -1;
	
	fprintf(stderr, "WAN=%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n",
		mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	return 0;
}

int show_wireless_mac()
{
	char mac[6];
	int err=0;
	
	err = read_mac(WIRELESS_MAC, mac);
	if(err) return -1;

	fprintf(stderr, "WIRELESS=%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n",
		mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	return 0;
}

void show_all_mac()
{
	show_lan_mac();
	show_wan_mac();
	show_wireless_mac();
}

int isHex(char c)
{
	if(c>='0' && c<='9')
		return 1;
	if(c>='A' && c<='F')
		return 1;
	if(c>='a' && c<='f')
		return 1;
	return 0;
}

// check mac if mac == xx:xx:xx:xx:xx:xx
int chk_input_mac(char *m)
{
	int i=0;

	if(strlen(m)!=17)
		return -1;
	
	if(m[2]!=':'||m[5]!=':'||m[8]!=':'||m[11]!=':'||m[14]!=':')
		return -1;
	
	for(i=0;i<5;i++){
		if( !isHex(m[i*3]) || !isHex(m[i*3+1]))
			return -1;
	}
	return 0;
}

int set_lan_mac(char *macstr)
{
	int err=0;
	unsigned char mac[6];

	fprintf(stderr, "set %s\n",macstr);

	if(chk_input_mac(macstr)==-1){
		fprintf(stderr, "MAC format should be xx:xx:xx:xx:xx:xx\n");
		return -1;
	}
	sscanf(macstr, "%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX",
	       mac, mac+1, mac+2, mac+3, mac+4, mac+5);

	fprintf(stderr, "LAN=%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n",
		mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	
	err = write_mac(LAN_MAC, mac);
	if(err) return -1;

	fprintf(stderr,"write LAN=%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n",
		mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	return 0;
}

int set_wan_mac(char *macstr)
{
	int err=0;
	unsigned char mac[6];

	fprintf(stderr, "set %s\n",macstr);

	if(chk_input_mac(macstr)==-1){
		fprintf(stderr, "MAC format should be xx:xx:xx:xx:xx\n");
		return -1;
	}
	sscanf(macstr, "%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX",
	       mac, mac+1, mac+2, mac+3, mac+4, mac+5);

	fprintf(stderr, "WAN=%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n",
		mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	
	err = write_mac(WAN_MAC, mac);
	if(err) return -1;

	fprintf(stderr,"write WAN=%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n",
		mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	return 0;
}
int set_wireless_mac(char *macstr)
{
	int err=0;
	unsigned char mac[6];

	fprintf(stderr, "set %s\n",macstr);

	if(chk_input_mac(macstr)==-1){
		fprintf(stderr, "MAC format should be xx:xx:xx:xx:xx\n");
		return -1;
	}
	sscanf(macstr, "%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX",
	       mac, mac+1, mac+2, mac+3, mac+4, mac+5);

	fprintf(stderr, "WIRELESS=%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n",
		mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	
	err = write_mac(WIRELESS_MAC, mac);
	if(err) return -1;

	fprintf(stderr, "write WIRELESS=%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n",
		mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	return 0;
}

void show_usage(char **argv)
{
	fprintf(stderr, "Usage: %s [read|write] [wan|lan|wireless|all] <mac address>\n",argv[0]);
	fprintf(stderr, "Description: read or wirte MAC address form/to flash\n");
	fprintf(stderr, "example: %s read wan (show wan mac address)\n", argv[0]);
	fprintf(stderr, "         %s read (show all mac address)\n", argv[0]);
	fprintf(stderr, "         %s read all (show all mac address)\n", argv[0]);
	fprintf(stderr, "         %s write wireless 00:12:34:56:78:9A (write wireless mac address)\n",
		argv[0]);
}


int mac_config_main(int argc, char **argv)
{
	switch(argc){
	case 2:
		if(!strcmp(argv[1],"write")){
			show_usage(argv);
			return -1;
		} else if(!strcmp(argv[1],"read")){
			show_all_mac();
		}
		break;
	case 3:
		if(!strcmp(argv[1],"read")){
			if(!strcmp(argv[2],"all")){
				show_all_mac();
			}else if(!strcmp(argv[2],"lan")){
				show_lan_mac();
			}else if(!strcmp(argv[2],"wan")){
				show_wan_mac();
			}else if(!strcmp(argv[2],"wireless")){
				show_wireless_mac();
			}else{
				show_usage(argv);
			}
		} else {
			show_usage(argv);
		}
		break;
	case 4:
		if(!strcmp(argv[1],"write")){
			if(!strcmp(argv[2],"lan")){
				set_lan_mac(argv[3]);
			}else if(!strcmp(argv[2],"wan")){
				set_wan_mac(argv[3]);
			}else if(!strcmp(argv[2],"wireless")){
				set_wireless_mac(argv[3]);
			}else{
				show_usage(argv);
			}
		} else {
			show_usage(argv);
		}
		break;

	default:
		show_usage(argv);
		return -1;
	}
	return 0;
}
