/* script.c
 *
 * Functions to call the DHCP client notification scripts 
 *
 * Russ Dill <Russ.Dill@asu.edu> July 2001
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/wait.h>
#include <errno.h>

#include "options.h"
#include "dhcpd.h"
#include "dhcpc.h"
#include "packet.h"
#include "options.h"
#include "debug.h"

#include <dirent.h>

#define READ_BUF_SIZE	50

#define A_DEBUG
#ifdef A_DEBUG
FILE *TTYS0;

#define DPRINTF(args...) \
do { \
     TTYS0=fopen("/dev/ttyS0","w"); \
     fprintf(TTYS0,"udhcp :"args); \
     fclose(TTYS0); \
} while (0); 
#else
#define DPRINTF(args...)
#endif

//static int
//udhcpc_show_packet(struct dhcpMessage *packet);
static char tmp_line[256];
static void
do1_extern_cmd(char *cmd, char *argv[], char *filename)
{
    char            path[100];
    int             pid,
                    fd = -1,
	fd2 = -1;

    fflush(stdout);
    signal(SIGCHLD, SIG_DFL);
    if ((pid = vfork()))
	waitpid(pid, NULL, 0);
    if (pid == 0) {		// child process
	if (filename != NULL && *filename) {
	    strcpy(path, filename);
	    fd = open(path, O_RDWR | O_CREAT | O_TRUNC);
	    strcat(path, ".err");
	    fd2 = open(path, O_RDWR | O_CREAT | O_TRUNC);
	}
	if (fd >= 0 && fd2 >= 0) {
	    dup2(fd, STDOUT_FILENO);
	    dup2(fd2, STDERR_FILENO);
	    execvp(cmd, argv);
	}
	exit(0);
    }
}
int
run_script_sh(char *filename, char *arg1, char *arg2, char *arg3)
{
    char           *argv[] = { "sh", filename, arg1, arg2, arg3, NULL };
    do1_extern_cmd("sh", argv, "/var/tmp/run_script");
    return 0;
}

void CopyFile(char *filename1,char *filename2){
	FILE *fp,*fp1;
	memset(tmp_line,0,sizeof(tmp_line));
	/* from filename1 to filename2 */ 
	if ((fp = fopen(filename1, "r"))!=NULL){
	  	if ((fp1 = fopen(filename2, "w"))!=NULL){
			while (fgets(tmp_line,sizeof(tmp_line),fp))
				fprintf(fp1,"%s",tmp_line);
			fclose(fp1);
		}
	fclose(fp);
	}
}

static int
udhcpc_write_status_file(struct dhcpMessage *, const char *);
char           *
read_file_para_value(char *filename, char *arg1,char *buf)
{
    FILE           *fp;
    char            line[255];
    char            str_return[50] = "" ;
    char            str_temp[50];
    if ((fp = fopen(filename, "r")) != NULL) {
	while (fgets(line, 255, fp)) {
	    if (strncmp(line, arg1, strlen(arg1)) == 0) {
		sscanf(line, "%50[^=]=%50s\n", str_temp, str_return);
		fclose(fp);
		strcpy(buf,str_return);
		return buf;
	    }
	}
    }
    return NULL;

}

int write_value_to_file(char *filename, int arg1)
{
	FILE *fp;
	if ((fp=fopen(filename,"w"))!=NULL)
        {
		fprintf(fp,"%d\n",arg1);
		fclose(fp);
		return 1;
	}
	else
		return 0;
	

}
int save_globe_config(char *filename, char *arg1, char *arg2)
{
//	char line[256];
//	int i,j;
	FILE *fp,*fp1;
//	i=0;
	memset(tmp_line,0,sizeof(tmp_line));	
	if((fp=fopen(filename,"r"))!=NULL){
		if((fp1=fopen("/var/tmp/tmpfile","w"))!=NULL){
			while(fgets(tmp_line,sizeof(tmp_line),fp))
			{
				if ((strncmp(arg1,tmp_line,strlen(arg1))) == 0){
				//	sprintf(config_line[i],"%s\"%s\"\n",arg1,arg2);
					fprintf(fp1,"%s\"%s\"\n",arg1,arg2);
				}
				else{
				//	sprintf(config_line[i],"%s",line);
					fprintf(fp1,"%s",tmp_line);
				}
	//		i++;
			}
	
		fclose(fp1);
	}
	fclose(fp);
	}
	CopyFile("/var/tmp/tmpfile",filename);
	return 1;	
}
int isValidIP(char *IPaddr){
 struct in_addr inp;
   if (inet_aton(IPaddr,&inp)!=0) /* success */
	   return 1;
   else
	   return 0;
}
long* KillProcessByName2( char* pidName){

	DIR *dir;
	struct dirent *next;
	long* pidList=NULL;
	int i=0;

	dir = opendir("/proc");
	if (!dir){
		DPRINTF("Cannot open /proc");
	}
	while ((next = readdir(dir)) != NULL) {
		FILE *status;
		char filename[READ_BUF_SIZE];
		char buffer[READ_BUF_SIZE];
		char name[READ_BUF_SIZE];

		/* Must skip ".." since that is outside /proc */
		if (strcmp(next->d_name, "..") == 0)
			continue;

		/* If it isn't a number, we don't want it */
		if (!isdigit(*next->d_name))
			continue;

		sprintf(filename, "/proc/%s/status", next->d_name);
		if (! (status = fopen(filename, "r")) ) {
			continue;
		}
		if (fgets(buffer, READ_BUF_SIZE-1, status) == NULL) {
			fclose(status);
			continue;
		}
		fclose(status);

		/* Buffer should contain a string like "Name:   binary_name" */
		sscanf(buffer, "%*s %s", name);
		if (strcmp(name, pidName) == 0) {
			pidList=realloc( pidList, sizeof(long) * (i+2));
			pidList[i++]=strtol(next->d_name, NULL, 0);
		}
	}

	if (pidList) {
		pidList[i]=0;
	} else {
		pidList=realloc( pidList, sizeof(long));
		pidList[0]=-1;
	}
	return pidList;
}
void KillProcessByName(char *ProcessName,int signal){
	
	long *killpid;
	
	killpid=KillProcessByName2(ProcessName);
	if (!killpid || *killpid<=0)
	{
		//DPRINTF(" %s : No Such Process\n",ProcessName);
	}
	else
	{
		//DPRINTF("kill -%d `pidof %s`\n",signal,ProcessName);
		kill(*killpid,signal);
	}
}

/* get a rough idea of how long an option will be (rounding up...) */
static int max_option_length[] = {
	[OPTION_IP] =		sizeof("255.255.255.255 "),
	[OPTION_IP_PAIR] =	sizeof("255.255.255.255 ") * 2,
	[OPTION_STRING] =	1,
	[OPTION_BOOLEAN] =	sizeof("yes "),
	[OPTION_U8] =		sizeof("255 "),
	[OPTION_U16] =		sizeof("65535 "),
	[OPTION_S16] =		sizeof("-32768 "),
	[OPTION_U32] =		sizeof("4294967295 "),
	[OPTION_S32] =		sizeof("-2147483684 "),
};


static int upper_length(int length, struct dhcp_option *option)
{
	return max_option_length[option->flags & TYPE_MASK] *
	       (length / option_lengths[option->flags & TYPE_MASK]);
}


static int sprintip(char *dest, char *pre, unsigned char *ip) {
//	return sprintf(dest, "%s%d.%d.%d.%d ", pre, ip[0], ip[1], ip[2], ip[3]);
	return sprintf(dest, "%s%d.%d.%d.%d ", pre, ip[0], ip[1], ip[2], ip[3]);
}
static int sprintip1(char *dest, char *pre, unsigned char *ip) {
//	return sprintf(dest, "%s%d.%d.%d.%d ", pre, ip[0], ip[1], ip[2], ip[3]);
	return sprintf(dest, "%s%d.%d.%d.%d", pre, ip[0], ip[1], ip[2], ip[3]);
}

/* Fill dest with the text of option 'option'. */
static void fill_options(char *dest, unsigned char *option, struct dhcp_option *type_p)
{
	int type, optlen;
	u_int16_t val_u16;
	int16_t val_s16;
	u_int32_t val_u32;
	int32_t val_s32;
	int len = option[OPT_LEN - 2];

	dest += sprintf(dest, "%s=", type_p->name);

	type = type_p->flags & TYPE_MASK;
	optlen = option_lengths[type];
	for(;;) {
		switch (type) {
		case OPTION_IP_PAIR:
			dest += sprintip(dest, "", option);
			*(dest++) = '/';
			option += 4;
			optlen = 4;
		case OPTION_IP:	/* Works regardless of host byte order. */
			dest += sprintip(dest, "", option);
 			break;
		case OPTION_BOOLEAN:
			dest += sprintf(dest, *option ? "yes " : "no ");
			break;
		case OPTION_U8:
			dest += sprintf(dest, "%u ", *option);
			break;
		case OPTION_U16:
			memcpy(&val_u16, option, 2);
			dest += sprintf(dest, "%u ", ntohs(val_u16));
			break;
		case OPTION_S16:
			memcpy(&val_s16, option, 2);
			dest += sprintf(dest, "%d ", ntohs(val_s16));
			break;
		case OPTION_U32:
			memcpy(&val_u32, option, 4);
			dest += sprintf(dest, "%lu ", (unsigned long) ntohl(val_u32));
			break;
		case OPTION_S32:
			memcpy(&val_s32, option, 4);
			dest += sprintf(dest, "%ld ", (long) ntohl(val_s32));
			break;
		case OPTION_STRING:
			memcpy(dest, option, len);
			dest[len] = '\0';
			return;	 /* Short circuit this case */
		}
		option += optlen;
		len -= optlen;
		if (len <= 0) break;
	}
}


static char *find_env(const char *prefix, char *defaultstr)
{
	extern char **environ;
	char **ptr;
	const int len = strlen(prefix);

	for (ptr = environ; *ptr != NULL; ptr++) {
		if (strncmp(prefix, *ptr, len) == 0)
			return *ptr;
	}
	return defaultstr;
}


/* put all the paramaters into an environment */
static char **fill_envp(struct dhcpMessage *packet)
{
	int num_options = 0;
	int i, j;
	char **envp;
	unsigned char *temp;
	char over = 0;

	if (packet == NULL)
		num_options = 0;
	else {
		for (i = 0; options[i].code; i++)
			if (get_option(packet, options[i].code))
				num_options++;
		if (packet->siaddr) num_options++;
		if ((temp = get_option(packet, DHCP_OPTION_OVER)))
			over = *temp;
		if (!(over & FILE_FIELD) && packet->file[0]) num_options++;
		if (!(over & SNAME_FIELD) && packet->sname[0]) num_options++;		
	}
	
	envp = malloc((num_options + 5) * sizeof(char *));
	envp[0] = malloc(sizeof("interface=") + strlen(client_config.interface));
	sprintf(envp[0], "interface=%s", client_config.interface);
	envp[1] = find_env("PATH", "PATH=/bin:/usr/bin:/sbin:/usr/sbin");
	envp[2] = find_env("HOME", "HOME=/");

	if (packet == NULL) {
		envp[3] = NULL;
		return envp;
	}

	envp[3] = malloc(sizeof("ip=255.255.255.255"));
	sprintip(envp[3], "ip=", (unsigned char *) &packet->yiaddr);
	for (i = 0, j = 4; options[i].code; i++) {
		if ((temp = get_option(packet, options[i].code))) {
			envp[j] = malloc(upper_length(temp[OPT_LEN - 2], &options[i]) + strlen(options[i].name) + 2);
			fill_options(envp[j], temp, &options[i]);
			j++;
		}
	}
	if (packet->siaddr) {
		envp[j] = malloc(sizeof("siaddr=255.255.255.255"));
		sprintip(envp[j++], "siaddr=", (unsigned char *) &packet->yiaddr);
	}
	if (!(over & FILE_FIELD) && packet->file[0]) {
		/* watch out for invalid packets */
		packet->file[sizeof(packet->file) - 1] = '\0';
		envp[j] = malloc(sizeof("boot_file=") + strlen(packet->file));
		sprintf(envp[j++], "boot_file=%s", packet->file);
	}
	if (!(over & SNAME_FIELD) && packet->sname[0]) {
		/* watch out for invalid packets */
		packet->sname[sizeof(packet->sname) - 1] = '\0';
		envp[j] = malloc(sizeof("sname=") + strlen(packet->sname));
		sprintf(envp[j++], "sname=%s", packet->sname);
	}	
	envp[j] = NULL;
	return envp;
}


/* Call a script with a par file and env vars */
void run_script1(struct dhcpMessage *packet, const char *name)
{
	int pid;
	char **envp;

	int is_wan_ip_the_same=0;
	if (client_config.script == NULL)
		return;

	/* call script */
	pid = vfork();
	if (pid) {
		waitpid(pid, NULL, 0);
		return;
	} else if (pid == 0) {
	  is_wan_ip_the_same=udhcpc_write_status_file(packet, name);
	  
	  	if(!strcmp(name,"deconfig") || !strcmp(name,"bound")){
	  		if (is_wan_ip_the_same==1&&!strcmp(name,"bound")){
			  LOG(LOG_INFO, "the renew ip is same with previous ip"); 
				exit(1);
			}
 			run_script_sh("/etc/scripts/fw_stop.sh", 0, 0, 0);
		}

		envp = fill_envp(packet);
		
		/* close fd's? */
		
		/* exec script */
		//DEBUG(LOG_INFO, "execle'ing %s", client_config.script);
 		DEBUG(LOG_INFO, "executing .....");
		execle(client_config.script, client_config.script,
		       name, NULL, envp);
		LOG(LOG_ERR, "executing failed ..");
		exit(1);
	}			
}




/* the two files for showing home.asp and running fw.sh */
static int
udhcpc_write_status_file(struct dhcpMessage *packet, const char *name)
{
  int optlen, type;
  int num_options = 0;
  int i;
  unsigned char *temp, *str;
  char over = 0;
  char str_wan_ip[20];
  char str_wan_ip_old[20];
  char str_wan_mask[20];
  char str_wan_gateway[20];
  char str_wanif[10];
  char str_dns[40];
  char str_dns1[20];
  char str_dns2[20];
  //char str_old_ip[20];
//  char str_old_if[10];
//  char is_fw[10];
  char buf[64];
//  char line[256];
  FILE *fp;
//  FILE *fp1;

  strcpy(str_wan_ip_old,(char *)read_file_para_value("/var/etc/status","WAN_IP=",buf));
  if (isValidIP(str_wan_ip_old)){
  //DPRINTF("old_wan_ip:%s\n", str_wan_ip_old);
     }
  DPRINTF("run in %s\n", name);
  if(strcmp(name, "deconfig")==0){
    save_globe_config("/var/etc/status","WAN_IP=","");
    save_globe_config("/var/etc/status","WAN_MASK=","");
    save_globe_config("/var/etc/status","GATEWAY=","");
    write_value_to_file("/tmp/ppp/wan_connect_status",0);
  }
  if( strcmp(name, "renew")!=0 && strcmp(name, "bound")!=0)
    return 0;
  else{	

	
  if (packet == NULL)
    num_options = 0;
  else {
    for (i = 0; options[i].code; i++)
      if (get_option(packet, options[i].code))
	num_options++;
    if (packet->siaddr) num_options++;
    if ((temp = get_option(packet, DHCP_OPTION_OVER)))
      over = *temp;
    if (!(over & FILE_FIELD) && packet->file[0]) num_options++;
    if (!(over & SNAME_FIELD) && packet->sname[0]) num_options++;		
  }

  if (packet == NULL) {
    return 0;
  }

	
  /* WAN_IF */
  sprintf(str_wanif, client_config.interface);
  DPRINTF("interface=%s\n", str_wanif);

  /* WAN_IP */
//  sprintip(str_wan_ip, "", (unsigned char *) &packet->yiaddr);
  sprintip1(str_wan_ip, "", (unsigned char *) &packet->yiaddr);
  DPRINTF("yiaddr=%s\n", str_wan_ip);

  unlink("/tmp/ppp/is_ip_changed");
  if (!strcmp(str_wan_ip,str_wan_ip_old)){	
  	write_value_to_file("/tmp/ppp/is_ip_changed",0);
	return 1;
     }
  
  write_value_to_file("/tmp/ppp/wan_connect_status",2);
  KillProcessByName("zebra",SIGTERM);
  KillProcessByName("zebra",SIGKILL);
  
  for (i = 0; options[i].code; i++) {
    if ((temp = get_option(packet, options[i].code))) {
      //DPRINTF("options name=%s\n", options[i].name);
      /* WAN_MASK */
      if(strcmp(options[i].name, "subnet")==0){
	//sprintip(str_wan_mask, "", temp);
	sprintip1(str_wan_mask, "", temp);
	DPRINTF("enter wan mask %s\n", str_wan_mask);
      }
      /* GATEWAY */
      if(strcmp(options[i].name, "router")==0){
	//sprintip(str_wan_gateway, "", temp);
	sprintip1(str_wan_gateway, "", temp);
	DPRINTF("enter gateway %s\n", str_wan_gateway);
      }
      /* DNS */
      
      if(strcmp(options[i].name, "dns")==0) {
	type = options[i].flags & TYPE_MASK;
	optlen = option_lengths[type];
//	DPRINTF("optlen=\"%d\"\n", optlen);
	str = str_dns;
	str += sprintip(str, "", temp);
	if(*(temp-1)>4){
	  temp+=4;
	  sprintip(str, "", temp);
	}
	DPRINTF("str_dns:%s\n",str_dns);
	sscanf(str_dns,"%20s %20s",str_dns1,str_dns2);
	if (isValidIP(str_dns1))
		DPRINTF("dns1 %s\n", str_dns1);
	if (isValidIP(str_dns2))
		DPRINTF("dns2 %s\n", str_dns2);
      }
    }
  }

  if (isValidIP(str_wan_gateway)){
    save_globe_config("/var/etc/status","WAN_IP=",str_wan_ip);
    save_globe_config("/var/etc/status","WAN_MASK=",str_wan_mask);
    save_globe_config("/var/etc/status","GATEWAY=",str_wan_gateway);
    save_globe_config("/var/etc/status","WAN_IF=",str_wanif);
  }
}
  return 0;

}


