/*
 *
 * Copyright 2001-2003, Broadcom Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: igd.c,v 1.1.1.5 2003/11/08 08:11:15 honor Exp $
 */

#include "upnp_dbg.h"
#include "upnp_osl.h"
#include "upnp.h"
#include "igd.h"
#include "bcmnvram.h"

#include <signal.h>

extern DeviceTemplate LANDeviceTemplate;
extern DeviceTemplate WANDeviceTemplate;

extern PDevice init_device(PDevice parent, PDeviceTemplate, ...);

extern char* read_file_para_value(char *filename, char *arg1,char *buf);
void sigusr1_handler(int i);

/* these should be set by OS-specific initialization code. */
char g_wandevs[20];
char g_landevs[20];
char str_lan_ip[20];
char str_goahead_port[10];

Error IGDErrors[] = {
    { SOAP_DISCONNECTINPROGRESS, "DisconnectInProgress" },
    { SOAP_INVALIDCONNECTIONTYPE, "InvalidConnectionType" },
    { SOAP_CONNECTIONALREADYTERMNATED, "ConnectionAlreadyTerminated" },
    { SOAP_CONNECTIONNOTCONFIGURED, "ConnectionNotConfigured" },
    { SOAP_SPECIFIEDARRAYINDEXINVALID, "SpecifiedArrayIndexInvalid" },
    { SOAP_NOSUCHENTRYINARRAY, "NoSuchEntryInArray" },
    { SOAP_CONFLICTINMAPPINGENTRY, "ConflictInMappingEntry" },
    { SOAP_ONLYPERMANENTLEASESSUPPORTED, "OnlyPermanentLeasesSupported" },
    { 0, NULL }
};
char           *
read_file_para_value(char *filename, char *arg1,char *buf)
{
    FILE           *fp;
    char            line[255];
    char            str_return[50] = "" ;
    char            str_temp[50];
    if ((fp = fopen(filename, "r")) != NULL) {
	while (fgets(line, 255, fp)) {
	    if (strncmp(line, arg1, strlen(arg1)) == 0) {
		sscanf(line, "%50[^=]=%50s\n", str_temp, str_return);
		fclose(fp);
		strcpy(buf,str_return);
		return buf;
	    }
	}
    }
    return NULL;

}

int IGDevice_Init(PDevice igdev, device_state_t state, va_list ap)
{
    char *wan_ifname = &g_wandevs[0];
    //char *lan_ifname = &g_landevs[0];
    PDevice subdev;

    switch (state) {
    case DEVICE_CREATE:
	soap_register_errors(IGDErrors);

	//igdev->friendlyname = "Residential Gateway Device";
	igdev->friendlyname = "Residential Home Gateway Device";

#if INCLUDE_LANDEVICE
	/* Include this code if you want a LAN device in your IGD.  If
	   you do, make sure that the LANHOSTCONFIG service is also included
	   in igd_desc.c */
	//fprintf(stderr,"Calling init_device for LANDevice\n");
	subdev = init_device(igdev, &LANDeviceTemplate, lan_ifname);
	subdev->next = igdev->subdevs;
	igdev->subdevs = subdev;
#endif
	
	/* for each wan interface, create a WANDevice */
	//fprintf(stderr,"Calling init_device for WANDevice\n");
	subdev = init_device(igdev, &WANDeviceTemplate, wan_ifname);
	subdev->next = igdev->subdevs;
	igdev->subdevs = subdev;

	/* install a signal handler for SIGUSR1 */
	signal(SIGUSR1, sigusr1_handler);

	break;

    case DEVICE_DESTROY:
	break;
    }
    return TRUE;
}


int WANDevice_Init(PDevice pdev, device_state_t state, va_list ap)
{
    PWANDevicePrivateData pdata;
    char *ifname = NULL;

    switch (state) {
    case DEVICE_CREATE:
	ifname = va_arg( ap, char *);
	if (ifname) {
	    pdata = (PWANDevicePrivateData) malloc(sizeof(WANDevicePrivateData));
	    if (pdata) {
		strcpy(pdata->ifname, nvram_safe_get("wan_ifname"));
		pdev->opaque = (void *) pdata;
	    }
	}
	break;

    case DEVICE_DESTROY:
	free(pdev->opaque);
	pdev->opaque = NULL;
	break;
    }

    return TRUE;
}


int LANDevice_Init(PDevice pdev, device_state_t state, va_list ap)
{
    PLANDevicePrivateData pdata;
    char *ifname = NULL;

    switch (state) {
    case DEVICE_CREATE:
	ifname = va_arg( ap, char *);
	if (ifname) {
	    pdata = (PLANDevicePrivateData) malloc(sizeof(LANDevicePrivateData));
	    if (pdata) {
		strcpy(pdata->ifname, ifname);
		pdev->opaque = (void *) pdata;
	    }
	}
	break;

    case DEVICE_DESTROY:
	free(pdev->opaque);
	pdev->opaque = NULL;
	break;
    }

    return TRUE;
}

int igd_config_generation = 0;

void bump_generation()
{
    if (++igd_config_generation > 1000)
	igd_config_generation = 0;
}

void sigusr1_handler(int i)
{
    extern void reinit_lan_interface(struct iface *pif);
    extern struct iface *global_lans;
    struct iface *pif;

    char buf[32];
//    char line[255];
    char wanif[10];
    printf("upnp reinitialize.\n");
    
    strcpy(wanif,(char *)read_file_para_value("/var/etc/status","WAN_IF=",buf));
    
    strcpy(g_wandevs, wanif);
    nvram_set("wan_ifname",wanif);
    bump_generation();

    for (pif = global_lans; pif; pif = pif->next) 
	reinit_lan_interface(pif);

    /* re-install a signal handler for SIGUSR1 */
    signal(SIGUSR1, sigusr1_handler);
}


