/*
 * Copyright 2001-2003, Broadcom Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: linux_main.c,v 1.1.1.5 2003/11/08 08:11:16 honor Exp $
 */

#include <errno.h>	    // for errno, of course.
#include <error.h>	    // for perror
#include <signal.h>	    // for signal, etc.
#include <assert.h>	    // for assert, of course.
#include <stdlib.h>	    // for malloc, free, etc.
#include <string.h>	    // for memset, strncasecmp, etc.
#include <stdarg.h>	    // for va_list, etc.
#include <stdio.h>	    // for printf, perror, fopen, fclose, etc.
#include <net/if.h>	    // for struct ifreq, etc.
#include <sys/ioctl.h>	    // for SIOCGIFCONF, etc.
#include <fcntl.h>	    // for fcntl, F_GETFL, etc.
#include <unistd.h>	    // for read, write, etc.
#include <arpa/inet.h>	    // for inet_aton, inet_addr, etc.
#include <time.h>	    // for time
#include <netinet/in.h>	    // for sockaddr_in
#include <wait.h>	    // for sockaddr_in


#include "ctype.h"
#include "upnp_dbg.h"
#include "upnp.h"

extern void define_variable(char *name, char *value);
extern void uuidstr_create(char *);
extern char *strip_chars(char *, char *);
extern void init_devices();
extern char* read_file_para_value(char *filename, char *arg1,char *buf);
extern int nvram_set(const char *name, const char *value);
void init_event_queue(int);

extern struct net_connection *net_connections;
extern int global_exit_now;
extern struct iface *global_lans;

static void
reap(int sig)
{
	pid_t pid;

	while ((pid = waitpid(-1, NULL, WNOHANG)) > 0)
	{
//fprintf(stderr,"Reaped %d\n", pid);
	}
}

void write_client_pid_file ();

int log_perror_client = 1;

void go_daemon ()
{
	static int state = 0;
	int pid;


	/* Only do it once. */
	if (state)
		return;
	state = 1;

	/* Stop logging to stderr... */
	log_perror_client = 0;

	/* Become a daemon... */
	if ((pid = vfork ()) < 0)
	{
		//fprintf(stderr,"Can't fork daemon: %m\n");
	}
	else if (pid)
		exit (0);
	/* Become session leader and get pid... */
	pid = setsid ();

	/* Close standard I/O descriptors. */
        close(0);
        close(1);
        close(2);

	write_client_pid_file ();
}
#define _PATH_UPNP_PID "/var/run/upnp.pid"
char *path_dhclient_pid = _PATH_UPNP_PID;



void write_client_pid_file ()
{
	FILE *pf;
	int pfdesc;

	pfdesc = open (path_dhclient_pid, O_CREAT | O_TRUNC | O_WRONLY, 0644);

	if (pfdesc < 0) {
		//fprintf(stderr,"Can't create %s: %m\n", path_dhclient_pid);
		return;
	}

	pf = fdopen (pfdesc, "w");
	if (!pf)
	{
		//fprintf(stderr, "Can't fdopen %s: %m\n", path_dhclient_pid);
	}
	else {
		//fprintf (pf, "%ld\n", (long)getpid ());
		fclose (pf);
	}
}


int main(int argc, char *argv[])
{
    extern char g_wandevs[];
   // extern char g_landevs[];
    extern DeviceTemplate IGDeviceTemplate;
//    char **argp = &argv[1];
    //char *wanif = NULL;
    //char *lanif = NULL;
    char str_temp[2];
    char buf[32];
    char line[255];
    char str_http_port[10];
    char lanif[10];
    char wanif[10];
    FILE *fp;
    int daemonize = 0;
	HTTP_PORT=2800;
//    while (argp < &argv[argc]) {
//	if (strcasecmp(*argp, "-L") == 0) {
		if (read_file_para_value("/var/etc/status","LAN_IF=",buf))
    			strcpy(lanif,(char *)read_file_para_value("/var/etc/status","LAN_IF=",buf));
		//fprintf(stderr, "UPnP lan_ifname:%s\t", lanif);
		//lanif = *++argp;
//	} 
//	else if (strcasecmp(*argp, "-W") == 0) {
		if (read_file_para_value("/var/etc/status","WAN_IF=",buf))
			strcpy(wanif,(char *)read_file_para_value("/var/etc/status","WAN_IF=",buf));
		
		//fprintf(stderr, "wan_ifname:%s\t", wanif);
		//wanif = *++argp;
		strcpy(g_wandevs, wanif);
		nvram_set("wan_ifname",wanif);
//	}
//	else if (strcasecmp(*argp, "-P") == 0) {
		if ((fp=fopen("/flash/upnpcfg.conf","r"))!=NULL) {
        		if(fgets(line,255,fp))
   				sscanf(line,"%s %s\n",str_temp,str_http_port);
	       		fclose(fp);
		}
		else
			strcpy(str_http_port,"2800");
		
		HTTP_PORT=atoi(str_http_port);
		//fprintf(stderr, "port:%d\n", HTTP_PORT);
		//HTTP_PORT = atoi(*++argp);
//	}  
//	else if (strcasecmp(*argp, "-D") == 0) {
//	    daemonize = 1;
//	}
//	argp++;
//  }

    init_event_queue(40);

    if (lanif == NULL || wanif == NULL) {
	//fprintf(stderr, "usage: %s -L lan_ifname -W wan_ifname -P tcp_port\n", argv[0]);
    } else {
#if !defined(__UCLIBC__) || defined(__UCLIBC_HAS_MMU__)
	if (daemonize && daemon(1, 1) == -1) {
	    perror("daemon");
	    exit(errno);
	}
#else
	if (daemonize)
	{    
		go_daemon();
//	perror("daemon not supported");
//
		}
#endif

	/* We need to have a reaper for child processes we may create.
	   That happens when we send signals to the dhcp process to
	   release an renew a lease on the external interface. */
	signal(SIGCHLD, reap);

	/* For some reason that I do not understand, this process gets
	   a SIGTERM after sending SIGUSR1 to the dhcp process (to
	   renew a lease).  Ignore SIGTERM to avoid being killed when
	   this happens.  */
	//	signal(SIGTERM, SIG_IGN);
	signal(SIGUSR1, SIG_IGN);

	//fprintf(stderr, "calling upnp_main\n");
	upnp_main(&IGDeviceTemplate, lanif);
    }
    
    return 0;
}
