
/****************************************************************************
 * standard include files
 ****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <linux/sockios.h>
#include <linux/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>

struct v_io_a {
        unsigned int request_cmd;
        unsigned int portmask;
        unsigned int phyaddr;
        unsigned int regid;
        unsigned int regvalue;
	unsigned int tagmask;
	unsigned int VID[4];
};
enum {
    SIOCSVLANPORT = SIOCDEVPRIVATE + 1,
    SIOCGVLANPORT,
    SIOCLAST
};



#ifdef LIBVER
#define MAINFUNC vpctrl_main
#else
#define MAINFUNC main
#endif

#define MAX_IFINDEX 255

/****************************************************************************
 * private function prototypes
 ****************************************************************************/
static void get_interfaces(int fd);
static void get_address(int fd, char *ifname);

/****************************************************************************
 * public functions defined here
 ****************************************************************************/


/****************************************************************************
 * Function Name  : main
 * Arguments      : argc
 *                  argv
 * Returns        : -1 fail, 0 success
 * Variables      : 
 * Modifies       : 
 * Description    : 
 ****************************************************************************/
int MAINFUNC(int argc, char *argv[])
{
	int fd;
        char* cmd=NULL;
	struct ifreq ifr;
	struct v_io_a request_args;
	unsigned int readback=0;
        if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
           fprintf(stderr, "FATAL:  Couldn't open a socket..go figure!\n");
           exit(2);
          }   
	memset(&ifr, 0, sizeof(struct ifreq));
   
   if ((argc < 3) ) {
      fprintf(stdout,"Expecting argc to be 3-5, inclusive.  Was: %d\n",argc);
      exit(1);
   }
   else {
      cmd = argv[1];
     }
 
      strcpy(ifr.ifr_name, "eth0"); 
    if (strcasecmp(cmd, "rst") == 0) {
     
      
      request_args.request_cmd=4;
      ifr.ifr_data=(char*)&request_args;

      if (ioctl(fd, SIOCSVLANPORT, &ifr) < 0)  	
         fprintf(stderr,"ERROR: trying to add VLAN \n");                 
     }
   if (strncasecmp(cmd, "vlan",4) == 0) {
     
      request_args.portmask=atoi(argv[2]);
      printf("portmask==%d\n",request_args.portmask);
      request_args.request_cmd=1;
      ifr.ifr_data=(char*)&request_args;

      if (ioctl(fd, SIOCSVLANPORT, &ifr) < 0)  	
         fprintf(stderr,"ERROR: trying to add VLAN \n");                 
     }
   
   if (strcasecmp(cmd, "miir") == 0) {
     
      request_args.phyaddr=atoi(argv[2]);
      request_args.regid=atoi(argv[3]);
      
      request_args.request_cmd=6;
      ifr.ifr_data=(char*)&request_args;
      if(request_args.regvalue==0 || ioctl(fd, SIOCGVLANPORT, &ifr)<0)
         printf("try to read lan mii reg fail\n");
      printf("reg %d.%d ==%x\n",request_args.phyaddr,request_args.regid,request_args.regvalue);                
     }

   if (strcasecmp(cmd, "miiw") == 0) {
     
      request_args.phyaddr=atoi(argv[2]);
      request_args.regid=atoi(argv[3]);
      request_args.regvalue=strtol(argv[4],NULL,16);
      printf("value==%lx\n",request_args.regvalue);
      request_args.request_cmd=5;
      ifr.ifr_data=(char*)&request_args;
      if(ioctl(fd, SIOCSVLANPORT, &ifr)<0)
         printf("try to write lan mii reg fail\n");
                      
     }

   if (strcasecmp(cmd, "active") == 0)
      {
        if ( strcasecmp(argv[2], "igmp") ==0)
        { 
           request_args.request_cmd=2;
           ifr.ifr_data=(char*)&request_args;
           if (ioctl(fd, SIOCSVLANPORT, &ifr) < 0) 	
               fprintf(stderr,"ERROR: trying to enable VLAN Tag \n");                 
        }
        if ( strcasecmp(argv[2], "no") ==0)
        { 
           request_args.request_cmd=3;
           ifr.ifr_data=(char*)&request_args;
           if (ioctl(fd, SIOCSVLANPORT, &ifr) < 0) 	
               fprintf(stderr,"ERROR: trying to disable VLAN Tag \n");                 
        }
      }
	return 0;
}

/****************************************************************************
 * Function Name  : get_interfaces
 * Arguments      : fd
 * Returns        : void
 * Variables      : 
 * Modifies       : 
 * Description    : 
 ****************************************************************************/
static void get_interfaces(int fd)
{
	int i;
	struct ifreq ifr;

	for (i = 1; i < MAX_IFINDEX; i++)
	{
		memset((void *)&ifr, '\0', sizeof(struct ifreq));
		ifr.ifr_ifindex = i;
	
		if (ioctl(fd, SIOCGIFNAME, &ifr) < 0)
		{
			return;
		}
		printf("%s ", ifr.ifr_name);
	}
}

/****************************************************************************
 * Function Name  : get_address
 * Arguments      : fd
 *                  ifname
 * Returns        : void
 * Variables      : 
 * Modifies       : 
 * Description    : 
 ****************************************************************************/
static void get_address(int fd, char *ifname)
{
	struct ifreq ifr;
	struct in_addr in;

	memset((void *)&ifr, '\0', sizeof(struct ifreq));
	strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
	if (ioctl(fd, SIOCGIFADDR, &ifr) < 0)
	{
		return;
	}
	in.s_addr = *((unsigned long *)&ifr.ifr_addr.sa_data[2]);
	printf("%s", inet_ntoa(in));
}
